package cc.fascinated.websocket.impl;

import cc.fascinated.model.token.ScoreSaberPlayerScoreToken;
import cc.fascinated.model.token.ScoreSaberScoreToken;
import cc.fascinated.model.token.ScoreSaberWebsocketDataToken;
import cc.fascinated.services.ScoreService;
import cc.fascinated.services.UserService;
import cc.fascinated.websocket.Websocket;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.TextMessage;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "ScoreSaber Websocket")
public class ScoreSaberWebsocket extends Websocket {
    /**
     * The Jackson deserializer to use.
     */
    private final ObjectMapper objectMapper;

    /**
     * The user service to use
     */
    private final UserService userService;

    /**
     * The score service to use
     */
    private final ScoreService scoreService;

    @Autowired
    public ScoreSaberWebsocket(@NonNull ObjectMapper objectMapper, @NonNull UserService userService, @NonNull ScoreService scoreService) {
        super("ScoreSaber", "wss://scoresaber.com/ws");
        this.objectMapper = objectMapper;
        this.userService = userService;
        this.scoreService = scoreService;
    }

    @Override
    @SneakyThrows
    public void handleMessage(@NonNull TextMessage message) {
        String payload = message.getPayload();
        if (payload.equals("Connected to the ScoreSaber WSS")) { // Ignore the connection message
            return;
        }

        ScoreSaberWebsocketDataToken response = this.objectMapper.readValue(payload, ScoreSaberWebsocketDataToken.class);
        if (!response.getCommandName().equals("score")) { // Ignore non-score messages
            return;
        }

        // Decode the message using Jackson
        ScoreSaberPlayerScoreToken scoreToken = this.objectMapper.readValue(response.getCommandData().toString(), ScoreSaberPlayerScoreToken.class);
        ScoreSaberScoreToken score = scoreToken.getScore();
        ScoreSaberScoreToken.LeaderboardPlayerInfo player = score.getLeaderboardPlayerInfo();

        // Ensure the player is valid
        if (!this.userService.isValidSteamId(player.getId())) {
            return;
        }
        scoreService.trackScoreSaberScore(scoreToken);
    }
}
